#!/bin/bash

set -eu

# Directory to store queued emails
QUEUE_DIR="/var/spool/sendmail-queue"
if [ ! -d "$QUEUE_DIR" ]; then
    mkdir -p "$QUEUE_DIR"
fi

# Maximum number of emails in the queue
MAX_QUEUE_SIZE=100

# Function to send an email to Gotify
send_to_gotify() {
    subject="$1"
    body="$2"

    set +e
    printf "%s" "$body" | /usr/bin/notify-gotify "$subject" > /dev/null
    RESULT=$?
    set -e
    return $RESULT
}

# Function to process the email queue
process_queue() {
    for email_file in "$QUEUE_DIR"/*.email; do
        [ -e "$email_file" ] || continue

        # Read the email content
        subject=$(grep -m 1 "^Subject:" "$email_file" | sed 's/^Subject: //')
        body=$(sed '1,/^$/d' "$email_file")  # Skip headers and read the body

        # Try to send the email to Gotify
        if send_to_gotify "$subject" "$body"; then
            # If successful, delete the email from the queue
            rm "$email_file"
            echo "Email sent to Gotify and removed from queue: $email_file"
        else
            echo "Failed to send email to Gotify: $email_file"
        fi
    done
}

# Function to add an email to the queue
add_to_queue() {
    local email_file="$QUEUE_DIR/email_$(date +%s).email"

    # Check if the queue is full
    if [ "$(ls -1 "$QUEUE_DIR"/*.email 2>/dev/null | wc -l)" -ge "$MAX_QUEUE_SIZE" ]; then
        # Discard the oldest email(s) to make room for the new one
        ls -1t "$QUEUE_DIR"/*.email | tail -n +$(($MAX_QUEUE_SIZE)) | xargs rm -f
    fi

    # Save the email to the queue
    cat - > "$email_file"
    echo "Email added to queue: $email_file"
}

# Main script logic
if [ -t 0 ]; then
    # If run interactively, print usage instructions
    echo "Usage: echo 'Subject: Test email' | $0"
    exit 1
else
    # Read the email from stdin (if any)
    if read -t 0; then
        email_content=$(cat -)

        # Extract the subject and body
        subject=$(echo "$email_content" | grep -m 1 "^Subject:" | sed 's/^Subject: //')
        body=$(echo "$email_content" | sed '1,/^$/d')

        # Try to send the email to Gotify
        if send_to_gotify "$subject" "$body"; then
            echo "Email sent to Gotify."
        else
            echo "Failed to send email to Gotify. Adding to queue."
            echo "$email_content" | add_to_queue
        fi
    else
        # If no email is provided, process the queue
        echo "No email provided. Processing the queue."
    fi

    # Process the queue (retry failed emails)
    process_queue
fi
